/***************************************************************************
*   Copyright (C) 2004 by karye                                           *
*   karye@users.sourceforge.net                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#pragma once

#include <ThreadWeaver/Job>
#include <QTextStream>

#include "common.h"
#include "portagefiles.h"

/**
* @class: LoadPackageKeywordsJob
* @short: Thread for loading packages keyword-unmasked by user.
*/
class LoadPackageKeywordsJob : public ThreadWeaver::Job
{
public:
	LoadPackageKeywordsJob() = default;

	void run( ThreadWeaver::JobPointer, ThreadWeaver::Thread* ) override {
		qDebug() << "Updating KEYWORDS DATABASE !!!!";
		DEBUG_LINE_INFO;

		// Collect all mask dependatoms
		QFileInfo fileInfo( KurooConfig::defaultFilePackageKeywords() );
		if( fileInfo.isDir() ) {
			qDebug() << KurooConfig::defaultFilePackageKeywords() << " is a dir";
			if( !mergeDirIntoFile( KurooConfig::defaultFilePackageKeywords() ) ) {
				return;
			}
		}
		QFile file( KurooConfig::defaultFilePackageKeywords() );
		QTextStream stream( &file );
		QStringList linesPackage;
		if ( !file.open( QIODevice::ReadOnly ) )
			qWarning() << "Parsing package.keywords. Reading: " << KurooConfig::defaultFilePackageKeywords();
		else {
			while ( !stream.atEnd() )
				linesPackage += stream.readLine();
			file.close();
		}

		// Something is wrong, no files found, get outta here
		if ( linesPackage.isEmpty() )
			return;

		//setStatus( "PackageKeywords", i18n("Collecting user package keywords...") );

		DbConnection* const m_db = KurooDBSingleton::Instance()->getStaticDbConnection();
		KurooDBSingleton::Instance()->singleQuery(	QStringLiteral("CREATE TEMP TABLE packageKeywords_temp ( "
													"idPackage INTEGER UNIQUE, "
													"keywords VARCHAR(255) );")
													, m_db);

		KurooDBSingleton::Instance()->singleQuery( QStringLiteral("BEGIN TRANSACTION;"), m_db );

		for ( QStringList::Iterator it = linesPackage.begin(), end = linesPackage.end(); it != end; ++it ) {

			// set the atom string
			QStringList tokens = (*it).split(u' ');
			QString package = tokens[0];

			if( !(*it).trimmed().startsWith( u'#' ) && !(*it).trimmed().isEmpty() ) {
				PortageAtom atom( package );
				if ( atom.isValid() ) {
					// extract this line's keywords
					QString keywords;
					QStringList::iterator tokenIterator = tokens.begin();
					++tokenIterator;
					while ( tokenIterator != tokens.end() ) {
						keywords += *tokenIterator + u' ';
						++tokenIterator;
					}
					if ( keywords.isEmpty() )
						keywords = u'~' + KurooConfig::arch();

					QString id = KurooDBSingleton::Instance()->singleQuery(
						QStringLiteral("SELECT id FROM package WHERE name = '") + atom.package() + QStringLiteral("' AND category = '") + atom.category() + QStringLiteral("' LIMIT 1;"), m_db );

					if ( id.isEmpty() )
						qWarning() << QStringLiteral("Load package keywords: Can not find id in database for package %1/%2.")
						.arg( atom.category(), atom.package() );
					else
						KurooDBSingleton::Instance()->insert( QStringLiteral(
							"INSERT INTO packageKeywords_temp (idPackage, keywords) VALUES ('%1', '%2');" )
										.arg( id, keywords ), m_db );
				}
				else
					qWarning() << QStringLiteral("Parsing package.keywords. Can not match package %1 in %2.").arg( *it,
						KurooConfig::defaultFilePackageKeywords() );
			}

		}
		file.close();
		KurooDBSingleton::Instance()->singleQuery( QStringLiteral("COMMIT TRANSACTION;"), m_db );

		// Move content from temporary table to installedPackages
		KurooDBSingleton::Instance()->singleQuery( QStringLiteral("DELETE FROM packageKeywords;"), m_db );
		KurooDBSingleton::Instance()->insert( QStringLiteral("INSERT INTO packageKeywords SELECT * FROM packageKeywords_temp;"), m_db );
		KurooDBSingleton::Instance()->singleQuery( QStringLiteral("DROP TABLE packageKeywords_temp;"), m_db );

		KurooDBSingleton::Instance()->returnStaticDbConnection( m_db );
		//setStatus( "PackageKeywords", i18n("Done.") );
// 		return;
// 	}
//
// 	virtual void completeJob() {
		PortageFilesSingleton::Instance()->refresh( PACKAGE_KEYWORDS_SCANNED );
	}
	Q_DISABLE_COPY(LoadPackageKeywordsJob)
};
